// Copyright {YYYY} Brian Newman. All rights reserved.
/*
	A few words about this application.
*/
package main

import (
	"database/sql"
	"log"
	"os"

	"cdmnky.io/dev/templates/web/v1/handlers"
	"cdmnky.io/dev/templates/web/v1/local.pkg/src/config"
	"cdmnky.io/dev/templates/web/v1/local.pkg/src/global"
	"cdmnky.io/dev/templates/web/v1/local.pkg/src/sitevars"

	"go.cdmnky.io/v2/net/assets"
	"go.cdmnky.io/v2/net/session"
	UI "go.cdmnky.io/v2/ui"

	_ "github.com/mattn/go-sqlite3"
	_ "go.cdmnky.io/v2/net/session/providers/memory"
)

var (
	appname = "template"
	version = "yyyy.mm.x"
	build   = "yyyy.mm.dd"

	ui *UI.UI

	globConfig       *config.Config
	globAssets       *assets.Assets
	globSessions     *session.Manager
	globDatabaseConn *sql.DB
	globSitevars     *sitevars.Sitevars
)

func main() {

	// -----------------------------------------------------------------------------
	ui = UI.New(appname, version, build)
	err := ui.Parse(os.Args)
	if err != nil {
		ui.Error(err)
		os.Exit(1)
	}

	// -----------------------------------------------------------------------------
	globAssets = assets.New("webassets", mapOfAssets) // in assets.go

	// Load config.json values
	// -----------------------------------------------------------------------------
	data, err := os.ReadFile(global.ConfigFileName)
	if err != nil {
		log.Fatal(err)
		os.Exit(1)
	}
	globConfig = config.New()
	err = globConfig.Parse(data)
	if err != nil {
		log.Fatal(err)
		os.Exit(1)
	}

	// -----------------------------------------------------------------------------
	globSessions, _ = session.New("memory", globConfig.Cookiename, 2147483647)
	go globSessions.GC()

	// -----------------------------------------------------------------------------
	globSitevars = &sitevars.Sitevars{
		Sitename:  globConfig.Sitename,
		Dbfile:    globConfig.Dbfile,
		Port:      globConfig.Port,
		Copyright: globConfig.Copyright,
	}

	// -----------------------------------------------------------------------------
	globDatabaseConn, err = sql.Open("sqlite3", globConfig.Dbfile)
	if err != nil {
		log.Fatal(err)
	}
	defer globDatabaseConn.Close()

	ui.Banner()

	(handlers.NewServer(
		ui,
		globDatabaseConn,
		globAssets,
		globSessions,
		globSitevars,
	)).Start()

	ui.Out()
}
